# -------------------------------------------------------------------------------------
# MKDoc::XML::Tree
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      This class is a base class to represent XML trees in memory. A tree can be
#      constructed using the output of XML::Parser in 'Tree' mode.
#
# -------------------------------------------------------------------------------------
package MKDoc::XML::Tree;
use MKDoc::Util::Text2HTML;
use XML::Parser;
use strict;
use 5.008_000;
use utf8;
use constant INDENT => "  ";

$::MKD_XML_Tree_NODES = {};
sub _NODES_() { return $::MKD_XML_Tree_NODES }


##
# $class->new ($tree);
# --------------------
#   $tree - A perl structure as XML::Parser would produced when
#   used in 'Tree' mode.
#
#   Recursively processes $tree and creates as many nodes
#   as required, creating a new MKDoc::XML::Tree node. Returns
#   the top node.
##
sub new
{
    my $class = shift;
    $class = ref $class || $class;
    
    # if the node is a text node, let's process ASAP
    if (@_ == 2 and $_[0] eq '0')
    {
	my $self = bless {
	    name => '0',
	    is_text => 1,
	    text => $_[1],
	}, $class;
	
	_NODES_->{$self->id} = $self;
	return $self;
    }
    
    # if there is only one argument, then it's a bit of text that
    # we need to parse into a tree
    my ($name, $second);
    if (@_ == 1)
    {
	my $data = shift;
	my $parser = new XML::Parser ( Style => 'Tree' );
	
	eval
	{
	    my $tree = $parser->parse ($data);
	    $name = $tree->[0];
	    $second = $tree->[1];
	};
	
	if (defined $@ and $@)
	{
	    die $@ . "\n\n$data";
	}
    }
    
    # otherwise it's a node which is not a text node
    else
    {
	$name = shift;
	$second = shift;
    }
    
    my $self = bless {}, $class;
    my $self_id = $self->id;
    _NODES_->{$self_id} = $self;
    
    $self->{name} = $name;
    my @children = @{$second};
    $self->{attributes} = shift (@children);
    $self->{children} = [];
    
    my @self_children = ();
    while (@children)
    {
	my $name = shift (@children);
	my $args = shift (@children);
	my $node = $class->new ($name, $args);
	my $node_id = $node->id;
	
	push @self_children, $node_id;
	$node->{parent} = $self;
    }
    
    $self->{children} = \@self_children;
    return $self;
}


##
# $self->id;
# ----------
#   Returns the ID of that MKDoc::XML::Tree node
##
sub id
{
    my $self = shift;
    return 0 + $self;
}


##
# $self->name;
# ------------
#   Returns the name of that node
##
sub name
{
    my $self = shift;
    return $self->{name};
}


##
# $self->children;
# ----------------
#   Returns all the children nodes for this MKDoc::XML::Tree node.
#   Returns an arrayref in scalar context, a list otherwise.
##
sub children
{
    my $self = shift;
    my @res = map { _NODES_->{$_} } @{$self->{children}};
    return wantarray ? @res : \@res;
}


##
# $self->text;
# ------------
#   Returns the text that this MKDoc::XML::Tree node represents.
##
sub text
{
    my $self = shift;
    return text2xml ($self->{text}) if ($self->{is_text});
    
    my $name = $self->{name};
    my $attr = $self->_attributes_text;
    $attr ||= '';
    
    if ($self->children)
    {
	my @res = ();
	if ($attr eq '') { push @res, "<$name>"       }
	else             { push @res, "<$name $attr>" }
	
	foreach my $child ($self->children) { push @res, $child->text }	
	push @res, "</$name>";
	return join '', @res;
    }
    else
    {
	if ($attr eq '') { return "<$name />"       }
	else             { return "<$name $attr />" }
    }
}


##
# $self->text_nice;
# -----------------
#   Nicely outputs the tree. Returns a SCALAR.
##
sub text_nice
{
    my $self = shift;
    my $indent = shift;
    $indent = 0 unless (defined $indent);
    
    if ($self->{is_text})
    {
	return "\n" . INDENT x $indent . text2xml ($self->{text});
    }
    
    my $name = $self->{name};
    my $attr = $self->_attributes_text;
    my @children = $self->children;
    unless (@children)
    {
	if ($attr eq '') { return "\n" . INDENT x $indent . "<$name />"       }
	else             { return "\n" . INDENT x $indent . "<$name $attr />" }
    }
    
    my @res = ();
    if ($attr eq '') { push @res, "\n" . INDENT x $indent . "<$name>"       }
    else             { push @res, "\n" . INDENT x $indent . "<$name $attr>" }
    
    foreach my $child (@children) { push @res, $child->text_nice ($indent + 1) }
    push @res, "\n" . INDENT x $indent . "</$name>";
    return join '', @res;
}


##
# $self->replace_with (@nodes);
# -----------------------------
#   @nodes - A list of MKDoc::XML::Tree nodes.
#
#   Replaces the current node with @nodes, unless $self
#   has no parent in which case nothing happens.
##
sub replace_with
{
    my $self = shift;
    my $parent = $self->{parent} || return;
    
    my $self_id = $self->id;
    my $parent_id = $parent->id;
    my @nodes = @_;
    
    # set the new parent for all @nodes
    foreach my $node (@nodes)
    {
	$node->{parent} = $parent;
    }
    
    my @parent_children = $parent->children;
    my @parent_new_children = ();
    
    # for each parent child, if the child is the same
    # as $self then replace it with @nodes, otherwise
    # push replace it with itself
    foreach my $child (@parent_children)
    {
	if ($self_id == $child->id)
	{
	    push @parent_new_children, map { $_->id } @nodes;
	}
	else
	{
	    push @parent_new_children, $child->id;
	}
    }
    
    $parent->{children} = \@parent_new_children;
    delete _NODES_->{$self_id};
}


##
# $self->next_node();
# -------------------
#   Returns the next node related to $self.
##
sub next_node
{
    my $self = shift;
    my @self_children = $self->children;
    
    # if the node has children, then node->next() should return
    # the first of its children
    if (scalar @self_children) { return $self_children[0] }
    
    # if the node has no children, then if it has siblings
    # then it should return the next of it's siblings unless
    # the node is the last sibling itself
    my $parent = $self->{parent} || return;
    my @parent_children = $parent->children;
    
    while (my $sibling = shift (@parent_children))
    {
	if ( ($sibling->id == $self->id) and 
	     (scalar @parent_children) )
	{
	    return shift (@parent_children);
	}
    }
    
    return $parent->next_node;
    return;
}


##
# $self->right_sibling;
# --------------------
#   Returns the right sibling node of that node.
##
sub right_sibling
{
    my $self = shift;
    my $parent = $self->{parent} || return;
    my @parent_children = $parent->children;
    my $self_id = $self->id;

    while (@parent_children)
    {
	my $child = shift (@parent_children);
	return shift (@parent_children)
	    if ( ($self_id == $child->id) and
		 (scalar @parent_children) );
    }
    
    return;
}


##
# $self->left_sibling;
# --------------------
#   Returns the left sibling node of that node.
##
sub left_sibling
{
    my $self = shift;
    my $parent = $self->{parent} || return;
    my @parent_children = $parent->children;
    my $self_id = $self->id;
    
    while (@parent_children)
    {
	my $child = pop (@parent_children);
	return pop (@parent_children)
	    if ( ($self_id == $child->id) and
		 (scalar @parent_children) );
    }
    
    return;
}


##
# $self->insert_before ($node);
# -----------------------------
#   $node - An XML::Tree node.
#
#   Inserts a sibling node before that node.
##
sub insert_before
{
    my $self = shift;
    my $parent = $self->{parent} || return;
    my $node = shift;
    
    my @parent_children = $parent->children;
    my @new_parent_children = ();
    my $self_id = $self->id;
    
    foreach my $child (@parent_children)
    {
	if ($self_id == $child->id)
	{
	    $node->{parent} = $parent;
	    push @new_parent_children, ($node->id, $self->id);
	}
	else
	{
	    push @new_parent_children, $child->id;
	}
    }
    
    $parent->{children} = \@new_parent_children;
}


##
# $self->insert_after ($node);
# -----------------------------
#   $node - An XML::Tree node.
#
#   Inserts a sibling node before that node.
##
sub insert_after
{
    my $self = shift;
    my $parent = $self->{parent} || return;
    my $node = shift;
    
    my @parent_children = $parent->children;
    my @new_parent_children = ();
    my $self_id = $self->id;
    
    foreach my $child (@parent_children)
    {
	if ($self_id == $child->id)
	{
	    $node->{parent} = $parent;
	    push @new_parent_children, ($self->id, $node->id);
	}
	else
	{
	    push @new_parent_children, $child->id;
	}
    }
    
    $parent->{children} = \@new_parent_children;
}


##
# $self->insert_first_inner ($node);
# ----------------------------------
#   $node - An XML::Tree node.
#
#   Inserts $node as the first inner node of $self
#   Does not do a thing if $node is text.
##
sub insert_first_inner
{
    my $self = shift;
    return if $self->{is_text};
    
    my $node = shift;
    $node->{parent} = $self;
    $self->{children} = [ $node->id, @{$self->{children}} ];
    return $self;
}


##
# $self->insert_last_inner ($node);
# ---------------------------------
#   $node - An XML::Tree node.
#
#   Inserts $node as the first inner node of $self
#   Does not do a thing if $node is text.
##
sub insert_last_inner
{
    my $self = shift;
    return if $self->{is_text};
    
    my $node = shift;
    $node->{parent} = $self;
    $self->{children} = [ @{$self->{children}}, $node->id ];
    return $self;
}


##
# $self->trim_spaces;
# -------------------
#   Removes all unnecessary whitespaces,
#   carriage returns, etc.
##
sub trim_spaces
{
    my $self = shift;
    if ($self->{is_text})
    {
	my $text = $self->{text};
	$text =~ s/^(\s|\n|\r)*//;
	$text =~ s/(\s|\n|\r)*$//;
	if ($text eq '')
	{
	    $self->delete;
	}
	else
	{
	    $self->{text} = $text;
	}
    }
    else
    {
	foreach my $child ($self->children)
	{
	    $child->trim_spaces;
	}
    }
}


##
# $self->delete;
# --------------
#   Deletes this node and all its child nodes.
##
sub delete
{
    my $self = shift;
    my $forget_parent_node_update = shift;
    my $self_id = $self->id;
    
    # if this node has a parent, and $forget_parent_false is
    # FALSE, then we should remove the parent reference to it
    my $parent = $self->{parent};
    if ( defined $parent and
	 not (defined $forget_parent_node_update and
	      $forget_parent_node_update) )
    {
	my @parent_children = @{$parent->{children}};
	my @new_parent_children = grep (!/^$self_id$/, @parent_children);
	$parent->{children} = \@new_parent_children;
    }
    
    delete _NODES_->{$self_id};
    for ($self->children) { $_->delete (1) }
}


##
# $self->_attributes_text;
# ------------------------
#   Returns the attributes for this node as a string
#   Returns an empty string if there's no attributes
##
sub _attributes_text
{
    my $self = shift;
    my @attr = ();
    foreach my $attr_name (sort keys %{$self->{attributes}})
    {
	my $attr_value = text2xml ($self->{attributes}->{$attr_name});
	push @attr, "$attr_name=\"$attr_value\"";
    }
    return join " ", @attr;
}


##
# $self->root;
# ------------
#   Returns the root node
##
sub root
{
    my $self = shift;
    my $self_parent = $self->{parent};
    return $self unless (defined $self_parent);
    return $self_parent->root;
}


##
# $self->all_nodes;
# -----------------
#   Returns all nodes for the tree $self is in
##
sub all_nodes
{
    my $self = shift;
    my $root = $self->root;
    return $root->_all_nodes;
}


##
# $self->_all_nodes;
# ------------------
#   Returns all nodes for $self
##
sub _all_nodes
{
    my $self = shift;
    return ($self, map { $_->_all_nodes } $self->children);
}


1;
