#!/usr/bin/perl -w
# -----------------------------------------------------------------------------
# Makefile.PL - Makefile generator for MKDoc
# -----------------------------------------------------------------------------
#
#       Author : Steve Purkis <spurkis@mkdoc.com>
#      Version : $Id: Makefile.PL,v 1.1.2.9 2002/12/17 14:44:20 jhiver Exp $
#    Copyright : (c) MKDoc Holdings Ltd, 2000.
#
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
# -----------------------------------------------------------------------------

use 5.008;
use ExtUtils::MakeMaker;
use File::Spec;

my %dependencies = read_modules_txt();
my $PREFIX       = argv_param ('PREFIX') || $ENV{PREFIX} || File::Spec->rel2abs ('.');

print "  +  using installation prefix:\t$PREFIX\n";

WriteMakefile(
	      'NAME'	     => 'MKDoc',
	      'VERSION_FROM' => 'MKDoc.pm',
	      'PREFIX'       => '/usr/local/mkdoc',
	      'PMLIBDIRS'    => [ 'MKDoc', 'flo', 'lib' ],
	      'PREREQ_PM'    => { %dependencies },
	      'dist'         => { COMPRESS => 'gzip -9f', SUFFIX => 'gz', },
	     );




#
# %deps = read_modules_txt
# ------------------------
# Scans MODULES.TXT for anything that looks like a Module dependency.
# Ignores comments (ie: anything after '# ...' ).  Ie:
#
#    Foo::Bar v5.6    # comments about dependency
#
# Uses v0.01 if no version number is found.  Returns a hash of the form:
#
#    { dependency_name => $version }
#
sub read_modules_txt {
    my $modules_txt = 'MODULES.TXT';
    my %modules;

    print "  +  getting required modules from $modules_txt\n";

    unless (open (MODULES, $modules_txt)) {
	warn "  !  warning: couldn't read from $modules_txt: $!\n";
	return ();
    }

    my $comment     = qr/#.*$/;
    my $module_name = qr/ (?<!\[) \w+ (?: \:\: \w+)* (?!\]) /x;
    my $module_ver  = qr/ (?<=\s) v? (\d+ (?: \.\d+)*) /x;

    while (my $line = <MODULES>) {
	$line       =~ s/$comment//; # strip comments
	if ($line   =~ / ($module_name) \s+ $module_ver? /x) {
	    $modules{$1} = $2 || '0.01';
	}
    }

    close MODULES;

    return %modules;
}


#
# $val = argv_param( $name )
# --------------------------
# Scans @ARGV for named param ala: PARAM=VALUE
#
sub argv_param
{
    my $param_name = shift;
    my ($string)   = grep /^$param_name=/, @ARGV;
    return unless (defined $string and $string);

    ($string) = $string =~ /^$param_name=(.*)/;
    return $string;
}



##
# Overriding MakeMaker
# -----------------------------------------------------------------------------
##

package MY;

use strict;
use warnings;

use Config qw/ %Config /;
use File::Spec;


##
# Adds 'test_config' target dependency to 'test' and 'testdb' targets.
##
sub test {
    my $self    = shift;
    my $targets = $self->SUPER::test (@_);

    $targets    =~ s/^(test (?:db)? \s+ :: \s+)/$1 test_config /mxg;
    $targets   .= $self->MY::test_config_target();

    return $targets;
}


##
# Adds the following make targets:
#    make cover
##
sub postamble {
    my $self    = shift || die "self not passed!";
    my $targets = '';

    $targets   .= $self->MY::cover_target();

    return $targets;
}


##
# Returns a target that lets you do a "make test_config"
# to run a configuration script before running tests.
##
sub test_config_target {
    my $self = shift;

    my $config_script    = File::Spec->catfile('tools', 'configure_site.pl');
    my $site_dir         = File::Spec->catdir('t', 'site');
    my $site_su_dir      = File::Spec->catdir('$(TEST_SITE_DIR)', 'su');
    my $site_config_dir  = File::Spec->catdir('$(TEST_SITE_DIR)', 'config');
    my $site_config_file = File::Spec->catfile('config', 'setup.cfg');
    my $mkdoc_dir = File::Spec->rel2abs ('.');

    my $target = <<END_OF_TARGET;

# --- Customized Makefile.PL test_config section:

# MKDoc Test Site variables:
TEST_SITE_DIR         = $site_dir
TEST_SITE_SU_DIR      = $site_su_dir
TEST_SITE_CONFIG_DIR  = $site_config_dir
TEST_SITE_CONFIG_FILE = $site_config_file

test_config: pure_all
	\$(MKPATH) $site_dir/config 
	\$(MKPATH) $site_dir/su
	\$(PERL) -I "$mkdoc_dir" -M"MKDoc::Site::Setup" -e 'configure ("$site_dir/config/init.cfg")' mkdoc_dir=$mkdoc_dir prefix=$site_dir
	\$(PERL) -I "$mkdoc_dir" -M"MKDoc::Site::Setup" -e 'setup_global_cfg ("$site_dir/config/init.cfg")'
	\$(PERL) -I "$mkdoc_dir" -M"MKDoc::Site::Setup" -e 'setup_deploy_db ("$site_dir/config/init.cfg")'

clean ::
	\$(RM_RF) \$(TEST_SITE_CONFIG_DIR) \$(TEST_SITE_SU_DIR)

# --- End of customized test_config section

END_OF_TARGET

    return $target;
}



##
# Returns a target that lets you do a "make cover"
# to display statisticts on test coverage of the codebase.
##
sub cover_target {
    my $self   = shift;
    my $target = <<'END_OF_TARGET';

# --- Customized Makefile.PL test coverage section:

cover:
	$(RM_RF) cover_db
	PERL5OPT=-MDevel::Cover $(MAKE) test || true
	cover -report html cover_db > /dev/null

clean ::
	$(RM_RF) cover_db

# --- End of customized test coverage section

END_OF_TARGET

    return $target;
}



__END__
