# -------------------------------------------------------------------------------------
# flo::Component
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2002
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      Discussion content component. Should let the user add hyperlinks to pages
#
# -------------------------------------------------------------------------------------
package flo::Component;
use MKDoc::XML::Dumper;
use Unicode::Normalize;
use flo::Standard qw /table cgi/;
use strict;
use Carp;


sub link
{
    my $self = shift;
    my @links = flo::Editor::_ETERNAL_()->access ( class => [ 'link' ] );
    
    my $self_title_lc = lc ($self->title());
    $self_title_lc =~ s/\s+/ /g;
    $self_title_lc =~ s/^\s+//;
    $self_title_lc =~ s/\s+$//;
    foreach my $link (@links)
    {
	my $link_title_lc = lc ($link->title());
	$link_title_lc =~ s/\s+/ /g;
	$link_title_lc =~ s/^\s+//;
	$link_title_lc =~ s/\s+$//;
	return $link if ($self_title_lc eq $link_title_lc);
    }
    
    return;
}


sub is_saved
{
    my $self = shift;
    return $self->{is_saved};
}


sub activate
{
    my $self  = shift;
    my $class = ref $self || $self;
    $class =~ s/^.*:://;
    $class = uc ($class);
    defined $ENV{"MKD__DISABLE_$class"} and return;
    return 1;
}


# Returns a string that helps identifying the type of the
# component.
sub type
{
    my $self = shift;
    my $class = ref $self || $self;
    $class =~ s/^.*\:\://;
    return lc ($class);
}


##
# $self->edit;
# ------------
#   Return the edit HTML chunk for this component
##
sub edit
{
    my $self = shift;
    my $type = $self->type();
    my $tmpl = new Petal ( file => "editor/$type", lang => $self->language() );
    return $tmpl->process ( self => $self );
}


##
# $self->block_name();
# --------------------
# Used for the templates that are used to edit this component.
##
sub block_name
{
    my $self = shift;
    return $self->{param_name};
}


##
# $self->set_block_name ($new_block_name);
# ----------------------------------------
# Used by flo::Editor to prefix this component fields with the
# proper prefix.
##
sub set_block_name
{
    my $self = shift;
    $self->{param_name} = shift;
}


##
# $self->id;
# ----------
#   Returns the identifier of the current component, which should
#   be the identifier of the document followed by a dash, followed
#   be the position of the component in the page. I.e. 1-3
##
sub id
{
    my $self = shift;
    my $parent_id = $self->parent->id;
    my $position  = $self->position;
    return "$parent_id-$position";
}


##
# $self->parent_id;
# -----------------
#   Returns the ID of the parent document which holds this component.
##
sub parent_id
{
    my $self = shift;
    return $self->{parent_id} || flo::Standard::current_document()->id();
}


##
# $self->parent;
# --------------
#   Returns the parent document which holds this component.
##
sub parent
{
    my $self = shift;
    my $document_table = table ('Document');
    return $document_table->get ($self->parent_id);
}


##
# $self->equals ($component);
# ---------------------------
#   $component - component to compare $self with.
#
#   Returns TRUE if $self and $component have the same id,
#   FALSE otherwise.
##
sub equals
{
    my $self = shift;
    my $component = shift;
    return $self->id eq $component->id;
}


##
# $self->ancestors;
# -----------------
#   Returns a list of ancestors for this component.
##
sub ancestors
{
    my $self = shift;
    my $parent = $self->parent;
    return (wantarray) ? () : [] unless (defined $parent);
    
    my @res = ($parent->ancestors, $parent);
    return (wantarray) ? @res : \@res;
}


##
# $self->position;
# ----------------
#   Returns the position of this component comparatively with
#   its siblings.
##
sub position
{
    my $self = shift;
    return $self->{position};
}


##
# $self->set_position ($pos);
# ---------------------------
#   $pos - new position to set
#
#   Sets the position of this component comparatively with
#   its siblings.
##
sub set_position
{
    my $self = shift;
    $self->{position} = shift;
}


##
# $self->path;
# ------------
#   Returns the path_info of this component.
##
sub path
{
    my $self = shift;
    return $self->parent->path . $self->name;
}


##
# $self->title;
# -------------
#   Returns the title of this component.
##
sub title
{
    my $self = shift;
    exists $self->{'title'} and return $self->{'title'};
    return $self->name;
}


##
# $self->root;
# ------------
#   Returns the document root.
##
sub root
{
    my $self = shift;
    return $self->parent->root;
}


##
# $self->is_only_child;
# ---------------------
#   Returns TRUE if this component has no sibling components,
#   FALSE otherwise.
##
sub is_only_child
{
    my $self = shift;
    my @components = $self->parent->components;
    return @components > 1;
}


##
# $self->extension;
# -----------------
#   Returns the extension to use for the http name of
#   this component. Defaults to 'html'.
##
sub extension { return 'html' }


##
# $self->uri;
# -----------
#   Returns the URI of this component
##
sub uri
{
    my $self = shift;
    my $cgix = cgi()->new;
    $cgix->delete ($_) for ($cgix->param);
    $cgix->path_info ( $self->path );
    while (@_)
    {
	my ($key, $val) = (shift, shift);
	$cgix->param ($key, $val);
    }
    return $cgix->self_url;
}


##
# $self->name;
# ------------
#   Returns the name associated with this component,
#   with the extension.
##
sub name
{
    my $self = shift;
    return $self->{uri_name};
}


##
# $self->generate_xml;
# --------------------
#   Serializes the object to XML
##
sub generate_xml
{
    my $self  = shift;
    my $cgi   = delete $self->{cgi};
    my $param = delete $self->{param_name};

    my $xml = pl2xml ($self);

    $self->{cgi} = $cgi;
    $self->{param_name} = $param;
    
    $xml =~ s/^<.*?<perl>/<perl>/sm;
    $xml = NFC ($xml);
    # u_mark (\$xml);
    return $xml;
}


##
# $self->parse_xml;
# -----------------
#   Parses the XML and sets the object attributes from the
#   parsed data
##
sub parse_xml
{
    my $self = shift;
    my $xml  = shift;
    my $obj  = xml2pl ($xml);
    $self->{$_} = $obj->{$_} for (keys %{$obj});
}


##                                                               #
#  ALL THESE DEFAULT TO THE PARENT DOCUMENT METHODS, BUT SHOULD  #
#  BE PROPERLY RE-IMPLEMENTED LATER ON                           #
#                                                               ##

sub contributors
{
    my $self = shift;
    return $self->parent->contributors;
}


sub editor_created_id
{
    my $self = shift;
    return $self->parent->editor_created_id;
}


sub editor_created
{
    my $self = shift;
    return $self->parent->editor_created;
}


sub editor_last_modified_id
{
    my $self = shift;
    return $self->parent->editor_last_modified_id;
}


sub editor_last_modified
{
    my $self = shift;
    return $self->parent->editor_last_modified;
}


sub date_created
{
    my $self = shift;
    return $self->parent->date_created;
}


sub date_created_w3c
{
    my $self = shift;
    return $self->parent->date_created_w3c;
}


sub date_last_modified
{
    my $self = shift;
    return $self->parent->date_last_modified;
}


sub date_last_modified_w3c
{
    my $self = shift;
    return $self->parent->date_last_modified_w3c;
}


sub align
{
    my $self = shift;
    return $self->parent->align;
}


sub align_opposite
{
    my $self = shift;
    return $self->parent->align_opposite;
}


sub direction
{
    my $self = shift;
    return $self->parent->direction;
}


# returns this object as embeddable HTML using Petal
sub as_xhtml
{
    my $self = shift;
    my $type = $self->type();
    my $template = new Petal
        file => "component/$type",
	lang => $self->language();
    
    $Petal::INPUT  = 'XML';
    $Petal::OUTPUT = 'XHTML';
    return $template->process (self => $self);
}


sub language
{
    my $self = shift;
    return $self->parent()->language();
}


##                              #
#  HORRIBLE DEPRECATED METHODS  #
#                              ##

sub html    { return "HTML METHOD IS DEPRECATED" }
sub to_html { return shift->html                 }


sub Align
{
    my $self = shift;
    carp ref $self . "::Align is deprecated";
    return $self->align;
}


sub Direction
{
    my $self = shift;
    carp ref $self . "::Direction is deprecated";
    return $self->direction;
}


1;
