# -------------------------------------------------------------------------------------
# flo::editor::Image
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      Image content component. Should let the user upload images
#
# -------------------------------------------------------------------------------------
package flo::editor::Image;
use MKDoc::Util::Text2HTML;
use MKDoc::Config;
use flo::Standard qw /cgi path_info raw_path_info session table template current_document/;
use flo::Editor;
use strict;

use base qw /flo::Component
	     flo::editor::Mixin::compute_name
	     flo::editor::Mixin::normalize_name
	     flo::editor::Mixin::upload_file
	     flo::editor::Mixin::hr_name
	     /;

##
# $self->internal_location();
# ---------------------------
# Returns the internal location associated with $self.
##
sub internal_location
{
    my $self = shift;
    return MKDoc::Config->IMAGE_HTTP_PATH . "/" . $self->{image};
}


sub preferred_extension
{
    my $self  = shift;
    my $file  = $self->{image} || return 'none';
    $file =~ s/^.*\///;
    my ($ext) = $file =~ /.*?\.(.*)/;
    $ext ||= $self->type();
    return $ext;
}


sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};

    # if CGI is defined, then we probably want to do some stuff
    if (defined $cgi)
    {
	# update alt field
	if (defined $cgi->param ($param_name . "_alt"))
	{
	    my $alt = $cgi->param ($param_name . "_alt");
	    $self->{title} = $alt;
	}
	else
	{
	    my $empty = "";
	    $self->{title} = $empty;
	}
	
	# if the upload field is not empty, then the user
	# just uploaded a new image
	if (defined $cgi->param ($param_name . "_upload"))
	{
	    $self->{is_new} = 1;
	    my $new_image = $cgi->param ($param_name);
	    if ($new_image)
	    {
		my $new_image_no_slash  = $new_image;
		$new_image_no_slash =~ s/^.*\///g;
		$new_image_no_slash =~ s/^.*\\//g;
		
		my $new_image_friendly  = $self->normalize_name ($new_image_no_slash);
		my $dir                 = $self->compute_name;
		my $full_dir            = MKDoc::Config->IMAGE_DISK_PATH . "/" . $dir;
		$class->upload_file ('image_disk_path', $dir, $new_image_friendly, $cgi->param ($param_name))
		    or return $self;

		
		# attempts to set a sensible 'alt' image title by looking at the CGI field,
		# then the image name, and then defaulting to 'image_title' by default
		# (which should NEVER happen)
		$self->{image} = "$dir/$new_image_friendly";
		$self->{title} =
		    $cgi->param ($param_name . "_alt")    ||
		    $class->hr_name ($new_image_friendly) ||
		    'image_title';
	    }
	}
	else
	{
	    $self->{image} = $cgi->param ($param_name . "_img");
	}
    }
    
    return $self;
}


sub generate_xml
{
    my $self = shift;
    my $data = "";
    if ($self->{image})
    {
	my $src = $self->{image};
	my $alt = text2html ($self->{title});
	my $chr_11 = quotemeta (chr (11));
	$alt =~ s/\n|\r|$chr_11/ /gsm;
	$alt =~ s/\s+/ /g;
	$data = qq |<img src="$src" alt="$alt" />|;
    }
    
    return $data;
}


sub parse_xml
{
    my $self = shift;
    my $line = shift;
    if ($line)
    {
	my ($img, $alt) = $line =~ /<img src=\"(.*?)\" alt=\"(.*)\" \/>/;
	$self->{image}  = $img;
	$self->{title}  = html2text ($alt) || $self->{image};
    }
    else
    {
	$self->{image} = "";
	$self->{title} = "";
    }
}


sub html
{
    my $self = shift;
    my $src  = MKDoc::Config->IMAGE_HTTP_PATH . "/" . $self->{image};
    return '<img src="' . $src . '" alt="' . text2html ($self->{title}) . '" />';
}


sub uri
{
    my $self = shift;
    return $self->is_saved() ?
        $self->SUPER::uri ( @_ ) :
	$self->URI (@_);
}


sub URI
{
    my $self = shift;
    my $src  = MKDoc::Config->IMAGE_HTTP_PATH . "/" . $self->{image};
    return $src;
}


sub description
{
    my $self = shift;
    return $self->{title};
}


sub title
{
    my $self = shift;
    return $self->{title};
}

1;
