# -------------------------------------------------------------------------------------
# flo::editor::Link
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      Link content component. Should let the user add hyperlinks to pages
#
# -------------------------------------------------------------------------------------
package flo::editor::Link;
use flo::Editor;
use flo::Standard qw /general_config cgi path_info raw_path_info session table template current_document/;
use MKDoc::CGI;
use MKDoc::Util::Text2HTML;
use Unicode::Normalize;
use strict;

use base qw /flo::Component
	     flo::editor::Mixin::compute_name
	     flo::editor::Mixin::normalize_name/;


sub preferred_extension { 'html' }


sub link
{
    return shift;
}


sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};
    if (defined $cgi)
    {
	if (defined $cgi->param ($param_name . "_add"))
	{
	    $self->{url} = "http://";
	    $self->{title} = "";
	    $self->{description} = "";
	}
	else
	{
	    my $cgi_url         = $cgi->param ($param_name . "_url")         || '';
	    my $cgi_title       = $cgi->param ($param_name . "_title")       || '';
	    my $cgi_description = $cgi->param ($param_name . "_description") || '';

	    $self->{url} = join ' ', split /(?:\n|\r)/, $cgi_url;
	    $self->{url} =~ s/^\s+//;
	    $self->{url} =~ s/\s+$//;
	    $self->{title} = join ' ', split /(?:\n|\r)/, $cgi_title;
	    $self->{title} =~ s/^\s+//;
	    $self->{title} =~ s/\s+$//;
	    $self->{description} = join ' ', split /(?:\n|\r)/, $cgi_description;
	    $self->{description} =~ s/^\s+//;
	    $self->{description} =~ s/\s+$//;	    
	}
    }
    else
    {
	$self->{url} = "";
	$self->{title} = "";
	$self->{description} = "";
    }
    
    return $self;
}


sub URI
{
    my $self = shift;
    return $self->{url};
}


sub full_url
{
    my $self = shift;
    my $url  = shift;
    return $url if ($url =~ /^(http|https|mailto|news|gopher):/);
    
    if ($url =~ /^\//)
    {
	return $ENV{SERVER_NAME} . $url;
    }
    else
    {
	return $ENV{SERVER_NAME} . $ENV{PATH_INFO} . $url;
    }
}


sub url
{
    my $self = shift;
    $self->{url} = shift || return $self->{url};
}


sub url_clean
{
    my $self = shift;
    my $url = $self->url;
    $url =~ s/\&(?!amp;)/&amp;/g;
    return $url;
}


sub title
{
    my $self = shift;
    $self->{title} = shift || return $self->{title};
}


sub description
{
    my $self = shift;
    $self->{description} = shift || return $self->{description};
}


sub generate_xml
{
    my $self = shift;
    my $url = text2xml ($self->{url});
    my $description = text2xml ($self->{description});
    my $title = text2xml ($self->{title});
    my $data = "<a href=\"$url\" title=\"$description\">$title</a>";
    
    # normalizes the xml to form C
    $data = NFC ($data);
    return $data;
}


sub parse_xml
{
    my $self = shift;
    my $line = shift;
    if ($line)
    {
	my ($url, $description, $title) = $line =~ /\<a href=\"(.*?)\" title=\"(.*?)\"\>(.*)\<\/a\>/;
	$self->{url} = xml2text ($url);
	$self->{description} = xml2text ($description);
	$self->{title} = xml2text ($title);
    }
    else
    {
	$self->{url} = "";
	$self->{description} = "";
	$self->{title} = "";
    }
}


sub html
{
    my $self  = shift;
    my $url   = $self->{url};
    my $title = text2html ($self->{title});
    my $desc  = text2html ($self->{description});
    return "<a href=\"$url\" title=\"$title\">$desc</a>";
}


sub to_html { return shift->html }


sub uri
{
    my $self = shift;
    my $url  = $self->url();
    return $url if ($url =~ /^[A-Za-z]+\:/);
    
    $url =~ /^\// and return do {
	my $cgix = flo::Standard::cgi()->new();
	for ($cgix->param()) { $cgix->delete ($_) }
	$cgix->path_info ($url);
	$cgix->self_url;
    };
    
    return do {
	my $cgix = flo::Standard::cgi()->new();
	my $path = flo::Standard::path_info();
	my $relpath = $url;
	$path = File::Spec->canonpath ($path . $relpath);
	$cgix->path_info ($path);
	$cgix->self_url;
    };
}


1;

