# -------------------------------------------------------------------------------------
# flo::plugin::Account::Generic
# -------------------------------------------------------------------------------------
#
#       Author : Steve Purkis <spurkis@mkdoc.com>
#    Copyright : (c) 2002, MKDoc Holdings Ltd.
#      Version : $Id: Generic.pm,v 1.10.2.16 2003/05/30 12:29:32 jhiver Exp $
#
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
# -------------------------------------------------------------------------------------

=head1 NAME

flo::plugin::Account::Generic - abstract Account plugin class [TEMPORARY]

=head1 SYNOPSIS

 # inheritance
 use flo::plugin::Account::Generic;
 use base qw/ flo::plugin::Account::Generic /;

 # see rest of documentation...

 # execute this plugin if activated:
 flo::plugin::Account::Details -> main();

=head1 DESCRIPTION

This abstract plugin defines generic methods for the Account::* plugins.
It will soon be combined with B<flo::Plugin>.

=cut


package flo::plugin::Account::Generic;

use flo::Object;
use flo::Standard qw /general_config cgi path_info raw_path_info session template current_document/;
use flo::HTTP::Header;
use flo::Record::Ticket;
use flo::Record::Editor;

use MKDoc::Mailer;

use URI;
use Carp;
use Petal;

use strict;
use 5.008_000;
use utf8;



# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 INHERITS FROM

B<flo::Object>

=cut

use base qw/ flo::Object /;




# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 CLASS VARIABLES

=head2 $VERSION

Version (from CVS revision string).

=cut

our $VERSION = (split(/ /, '$Revision: 1.10.2.16 $'))[1];



# -------------------------------------------------------------------------------------

=head2 $DEBUG

Boolean switch controlling debug printing to STDERR.

=cut

our $DEBUG = 0;



# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 CLASS METHODS

=head2 $bool = $class->main()

Checks to see if this request should be activated, and processes the request
if so, or does nothing if not.

Returns 'TERMINATE' if the request was handled by this plugin, false if not,
or undef on error.

=cut

sub main ($)
{
    my $class  = shift || return;

    my $plugin = $class->new(@_) || return;

    # if this plugin is not concerned, return ASAP
    return unless $plugin->activate;

    return $plugin->process_request;
}



# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 CONSTRUCTOR

=head2 $obj = new $class( @args )

Create a new plugin of defined class.  @args are passed to the plugin's
_initialize() method.

Returns new object, or undef on error.

=cut

sub new
{
    my $class = shift || return;
    $class    = ref $class || $class;
    my $self  = bless {}, $class;
    return $self->_initialize(@_);
}



# -------------------------------------------------------------------------------------

=head2 $class->_initialize(@args)

Initializes this object, called by new().  Does nothing by default.
Returns this object.

=cut

sub _initialize
{
    my $self = shift || return;
    return $self;
}




# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 CLASS/INSTANCE METHODS

=head2 $bool = $class->activate()

Checks to see if this plugin should be activated.  It will be activated if the
HTTP request path contains this plugins I<http_n*ame> (as defined in this plugin's
config file).

Activation information is made available through the fuzzy means of flo::Standard's
C<raw_path_info()>, rather than explicitly as a parameter to this method.

Returns true if this plugin should be activated, false if not, undef on error.

=cut

sub activate ($)
{
    my $class = shift || return;

    warn "  i  checking $class plugin activation\n" if ($DEBUG);

    # get the http_name for this plugin from config file:
    my $http_name   = $class->name;
    my $q_http_name = quotemeta( $http_name );
    
    # check it against the raw path info of the HTTP request:
    my $raw_path_info = raw_path_info();
    if ($raw_path_info =~ /\/$q_http_name$/)
    {
	return 1;
    }
    else
    {
	warn "  i  http: $q_http_name     path: $raw_path_info\n" if ($DEBUG > 1);
	return 0;
    }
}



# -------------------------------------------------------------------------------------

=head2 $result = $class->process_request()

B<Abstract> method to process request after activation.  Calls select_state()
to select state & render the document.  Successful processing will result in
the document being printed to STDOUT.

Returns 'TERMINATE' if the request was handled by this plugin, false if not,
or undef on error.

=cut

sub process_request ($)
{
    my $class = shift || return;
    my $cgi   = cgi();
    my $doc   = $class->select_state($cgi);

    # print the rendered document & return
    if ($doc)
    {
	print $doc;
	return 'TERMINATE';
    }
    else
    {
	return 0;
    }
}



# -------------------------------------------------------------------------------------

=head2 $doc = $class->select_state( $cgi )

B<Abstract> method to select current state given the MKDoc::CGI object.  A valid
$document should be returned.

Returns a valid text $document on success, or undef on error.

=cut

sub select_state ($$)
{
    my $class = shift || return;
    confess "  !  $class does not override select_state()!";
}



# -------------------------------------------------------------------------------------

=head2 $ticket = $class->validate_ticket_id( $id )

Returns the ticket object with this ticket id, or undef on error.

=cut

sub validate_ticket_id ($$)
{
    my $class     = shift || return;
    my $ticket_id = shift || return;

    # clear expired tickets:
    flo::Record::Ticket->clear_old_tickets_from_db;

    # try to load ticket from db
    my $ticket = new flo::Record::Ticket($ticket_id);

    return $ticket;
}



# -------------------------------------------------------------------------------------

=head2 $doc = $class->render_http_template( @args )

Calls $class->render_template() with the arguments given, and returns the
results wrapped in an HTTP response.

Returns the rendered document on success, or undef on error.

=cut

sub render_http_template (@)
{
    my $class = shift || return;

    # render the template
    my $data  = $class->render_template(@_) || return;

    warn "  +  wrapping rendered template w/HTTP headers\n" if ($DEBUG);

    # Create the HTTP header
    my $header = new flo::HTTP::Header;
    $header->set( 'Content-Type: text/html; charset=UTF-8' );

    {
	no utf8;
	use bytes;
	$header->set( "Content-Length: " . length ($data) );
    }

    # return the rendered template
    return ($header->header . $data);
}



# -------------------------------------------------------------------------------------

=head2 $doc = $class->render_template( $template_name, $lang [, @processing_args ] )

Internal method to render the petal template specified by C<$template_name>
and C<$lang>.  Any C<@processing_args> given are passed into Petal's
C<process()> method.

Uses Carp's confess() to report fatal errors.

Returns the rendered document on success, or undef on error.

=cut

sub render_template ($$$@)
{
    my $class            = shift || return;
    my $template_name    = shift || return;
    my $lang             = shift || return;
    my %processing_args  = @_;

    warn "  +  rendering [$template_name]:[$lang] template\n" if ($DEBUG);

    # Figure out which template to use
    my $template = template($template_name, $lang);

    $class->_setup_petal( $template ) || confess "error setting up Petal!";

    # Create a new petal rendering machine & process the template
    my $petal = new Petal( $template );
    my $data  = $petal->process(
				$class->default_processing_args,
				%processing_args,
			       ) || confess "petal returned empty template!";

    # return the rendered template
    return $data;
}



# -------------------------------------------------------------------------------------

=head2 $doc = $thing->_setup_petal( $template )

Internal method to set petal processing arguments and input mode.
Returns true on success, or undef on error.

=cut

sub _setup_petal
{
    my $thing    = shift;
    my $template = shift;

    my $general_config   = general_config;
    $Petal::BASE_DIR     = $general_config->get('template_dir');
    $Petal::DISK_CACHE   = 0;
    $Petal::MEMORY_CACHE = 0;
    $Petal::OUTPUT       = 'XHTML';

    # if we're rendering a text file, *don't* use xml input mode:
    if ($template =~ /.txt$/i)
    {
	$Petal::INPUT    = 'XHTML';
    }
    else
    {
	$Petal::INPUT    = 'XML';
    }

    return $thing;
}



# -------------------------------------------------------------------------------------

=head2 $bool = $class->render_and_send_email( %args )

Renders the named email template and attempts to send it.  Valid args
(* are required):

 *  To               to email
    From             from email
    Bcc              bcc email
 *  Subject          subject
 *  Template         tmplate file
    ProcessingArgs   %hash of Petal processing args
 *  Errors           MKDoc::Error object

Attempts to get From, Bcc from MKDoc::Config if not set.

Sets these MKDoc::Error errors:
    mail.to.not_set
    mail.from.not_set
    mail.render
    mail.send

Returns the true on success, or undef on error.

=cut

sub render_and_send_email ($%)
{
    my $class  = shift || return;
    my %args   = @_;
    my $errors = delete $args{Errors};

    # check to:
    unless ($args{To})
    {
	warn "  !  user does not have an email!?";
	$errors->set('mail.to.not_set') if $errors;
	return 0;
    }

    warn "  +  composing email for $args{To}...\n" if ($DEBUG);

    # get from, bcc from config if necessary
    my $config       = new MKDoc::Config('flo.plugin.Account');
    $args{From}    ||= $config->get('mail_from');
    $args{Bcc}     ||= $config->get('mail_bcc');
    $args{Subject} ||= '[no subject]';

    unless ($args{From})
    {
	warn "  !  'mail_from' not set in plugin config file!";
	$errors->set('mail.from.not_set') if $errors;
	return 0;
    }

    # build the list of processing args available:
    $args{ProcessingArgs} = {} unless (exists $args{ProcessingArgs});

    # render the email template
    my $body = $class->render_template($args{Template},
				       $class->lang,
				       %{$args{ProcessingArgs}});

    unless (defined($body) and ($body))
    {
	warn "  !  error rendering [$args{Subject}] email!";
	$errors->set('mail.render') if $errors;
	return 0;
    }


    # create & send the email:
    my $mail = new MKDoc::Mailer;
    $mail->set_to($args{To})
         ->set_from($args{From})
	 ->set_subject($args{Subject})
	 ->set_body($body);
    $mail->set_mail_header('Bcc', $args{Bcc}) if ($args{Bcc});

    eval { $mail->send };

    if ($@)
    {
	warn "  !  error sending [$args{Subject}] email: $@!";
	$errors->set('mail.send', $@) if $errors;
	return 0;
    }

    return(1);
}



# -------------------------------------------------------------------------------------

=head2 $user = $thing->user()

Returns the currently logged-in user, or undef.

=cut

sub user ($)
{
    my $self = shift || return;

    # get cached copy?
    return $self->{_user_} if ($self->is_an_object and $self->{_user_});

    my $session = session;

    warn "  +  retrieving Editor ID from session\n" if ($DEBUG);

    unless ($session and exists($session->{Editor_ID}))
    {
	warn "  !  couldn't find Editor ID in session!\n" if ($DEBUG);
	return;
    }

    my $user = load flo::Record::Editor( ID => $session->{Editor_ID} );

    # FIXME: track down this bug (the commented line is right, but it always breaks?!):
    #unless ($user) {
    unless ($session and exists($session->{Editor_ID}))
    {
	warn "  !  couldn't load User from session [$session->{Editor_ID}]: $!\n";
	return;
    }

    # cache user obj
    $self->set_user($user) if ($self->is_an_object);

    return $user;
}



# -------------------------------------------------------------------------------------

=head2 $obj = $obj->set_user($user)

Set the '_user_' attribute to $user.  Returns this object.

=cut

sub set_user
{
    my $self        = shift;
    $self->{_user_} = shift;
    return $self;
}



# -------------------------------------------------------------------------------------

=head2 %args = $thing->default_processing_args()

Creates a hash of default processing args for use with Petal.
If $thing is not an object, this is empty.  Otherwise currently:

    self   => $this_object
    plugin => $this_object
    object => $parent_document

Returns the hash, or undef on error.

=cut

sub default_processing_args ($)
{
    my $thing = shift || return;
    my %args  = ();

    if ($thing->is_an_object) {
	%args = (
		 self   => $thing,
		 plugin => $thing,
		 object => $thing->parent,
		);
    }

    return %args;
}



# -------------------------------------------------------------------------------------

=head2 $file = $class->template_file()

B<Abstract> method.
Returns the template file associated with this class.

=cut

sub template_file ($)
{
    my $class = shift || return;
    confess "  !  $class does not override template_file()!";
}



# -------------------------------------------------------------------------------------

=head2 $url = $class->login_url()

Gets an explicitly defined login url from config, or falls back onto
admin_url.

Returns the url, or undef on error.

=cut

sub login_url ($)
{
    my $class        = shift || return;
    my $login_url    = general_config->get('login.url');
    $login_url     ||= $class->admin_url;
    return $login_url;
}




# -------------------------------------------------------------------------------------

=head2 $url = $class->admin_url()

Calculates the admin url for this MKDoc site, by taking the current
CGI request and substituting the admin domain (from config) in for
the hostname.

Returns the url, or undef on error.

=cut

sub admin_url ($)
{
    my $class        = shift || return;
    my $login_domain = general_config->get('admin.domain');
    my $cgi_url      = cgi->url( -base => 1 );

    # preserve port & scheme of current request
    my $uri = new URI($cgi_url);
    $uri->host($login_domain);
    $uri->path('/');

    return $uri->as_string;
}




# -------------------------------------------------------------------------------------

=head2 $url = $class->plugin_url( $plugin )

Gets the uri of the named $plugin (a plugin object, a plugin
class, or an MKDoc::Config key).

This will not work accross mkdoc virtual hosts (editor/su/mkdoc instances).

Returns the uri, or undef on error.

=cut

sub plugin_url ($$)
{
    my $class  = shift || return;
    my $plugin = shift || return;

    warn "  +  getting plugin url for $plugin\n" if ($DEBUG);

    # make sure we've got plugin MKDoc::Config can understand:
    $plugin = ref($plugin) || $plugin;
    $plugin =~ s/\:\:/\./g;

    # get httpname from config
    my $http_name = $class->name()
      || warn("  !  no http name found for $plugin") && return;

    # construct the uri from CGI obj
    my $path_info = current_document()->path . $http_name;
    my $cgix      = cgi->new();
    $cgix->delete_all();
    $cgix->path_info ($path_info);

    return $cgix->self_url;
}



# -------------------------------------------------------------------------------------

=head2 $class = $thing->class()

Returns the class of this $thing, or undef on error.

=cut

sub class ($)
{
    my $thing = shift || return;
    return ref($thing) || $thing;
}



# -------------------------------------------------------------------------------------

=head2 $bool = $thing->is_an_object()

Returns true if this $thing is an object, false if not.

=cut

sub is_an_object
{
    return ref(shift) ? 1 : 0;
}



# -------------------------------------------------------------------------------------

=head2 $thing->version()

Returns the value of $VERSION in this $thing's class, or undef if not set.

=cut

sub version ($)
{
    my $class = shift->class;
    {
	no strict 'refs';
	return ${$class . '::VERSION'};
    }
}



# -------------------------------------------------------------------------------------

=head2 $thing->debug( @text )

Prints @text out with warn(), if class $DEBUG variable is set.
Returns 1 if text was printed, undef if not.

=cut

sub debug ($@)
{
    my $class = shift->class;
    return unless($class->_debug_level);
    push @_, "\n" unless( grep(/\n/, @_) );
    warn @_;
    return 1;
}




# -------------------------------------------------------------------------------------

=head2 $thing->_debug_level()

Returns the value of $DEBUG in this $thing's class, or undef if not set.

=cut

sub _debug_level ($)
{
    my $class = shift->class;
    {
	no strict 'refs';
	return ${$class . '::DEBUG'};
    }
}


# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 STANDARD PLUGIN METHODS

=head2 $parent = $self->parent()

B<Taken from flo::Plugin.>

Returns the parent document object of the current object.
If the current object has no parent (in which case it's the front page
document), returns undef.

=cut

sub parent
{
    my $self     = shift;
    my $document = current_document();
    return $document;
}



# -------------------------------------------------------------------------------------

=head2 @ancestors = $obj->ancestors()

B<Taken from flo::Plugin.>

Returns the ancestors of this plugin, i.e. its parent, grand-parent,
grand-grand parent, etc...  the root element being the first element
of the array which is being returned.

Returns an array reference in scalar context, or a list otherwise.

=cut

sub ancestors
{
    my $self   = shift;
    my $parent = $self->parent;
    return (wantarray) ? () : [] unless (defined $parent);

    my @res = ($parent->ancestors, $parent);
    return (wantarray) ? @res : \@res;
}



# -------------------------------------------------------------------------------------

=head2 $lang = $class->lang()

B<Taken from flo::Plugin.>

Determines which template language to use by inherited the value from the
current document.

Returns the template language to use, or undef on error.

=cut

sub lang ($)
{
    my $class = shift || return;
    return current_document -> lang; # inherit language from current document
}



# -------------------------------------------------------------------------------------

=head2 $lang = $obj->title()

B<Taken from flo::Plugin.>

Returns the title of this document.

=cut

sub title
{
    my $self = shift;
    return $self->parent->title;
}



# -------------------------------------------------------------------------------------

=head2 $lang = $obj->lang_label()

B<Taken from flo::Plugin.>

Returns the current language label for this plugin, i.e. 'British English'

=cut

sub lang_label
{
    my $self   = shift;
    my $lang   = $self->lang;
    my $config = new MKDoc::Config ('languages');
    return $config->get ($lang);
}



# -------------------------------------------------------------------------------------

=head2 $lang = $obj->align()

B<Taken from flo::Plugin.>

Get the text alignment stlye for this plugin,  depending on the language.
Returns 'right' or 'left'.

=cut

sub align
{
    my $self   = shift;
    my $config = new MKDoc::Config ('languages_rtl');
    if (defined $config->get ($self->lang)) { return 'right' }
    else                                    { return 'left'  }
}



# -------------------------------------------------------------------------------------

=head2 $lang = $obj->align_opposite

B<Taken from flo::Plugin.>

Returns the opposite of $obj->align.

=cut

sub align_opposite
{
    my $self = shift;
    return ($self->align eq 'right') ? 'left' : 'right';
}



# -------------------------------------------------------------------------------------

=head2 $lang = $obj->direction

B<Taken from flo::Plugin.>

Returns the direction depending on the language, i.e. 'rtl' or 'ltr'

=cut

sub direction
{
    my $self   = shift;
    my $config = new MKDoc::Config ('languages_rtl');
    if (defined $config->get ($self->parent->lang)) { return 'rtl' }
    else                                            { return 'ltr' }
}



# -------------------------------------------------------------------------------------

=head2 $lang = $obj->uri( %cgi_params )

B<Taken from flo::Plugin.>

Returns the URI of this object with the CGI params given.

=cut

sub uri
{
    my $self = shift;
    my $cgix = cgi->new;
    $cgix->delete_all();
    while (@_)
    {
	my ($key, $val) = (shift, shift);
	$cgix->param ($key, $val);
    }
    $cgix->path_info ($self->path);
    return $cgix->self_url;
}



# -------------------------------------------------------------------------------------

=head2 $lang = $obj->path

B<Taken from flo::Plugin.>

Returns the path of this object, i.e. /foo/bar/plugin.html

=cut

sub path
{
    my $self   = shift;
    my $parent = $self->parent;
    return $self->parent->path . $self->name;
}



# -------------------------------------------------------------------------------------


sub name
{
    my $self = shift;
    
    my $env_name = $self->_name_env();
    my $def_name = $self->_name_default();
    return $ENV{$env_name} || $def_name;
}


sub _name_env
{
    my $class = shift;
    $class = ref $class || $class;
    $class =~ s/^flo::plugin:://;
    $class =~ s/\.pm$//;
    $class = uc ($class);
    $class =~ s/::/_/g;
    return "MKD__" . $class . "_HTTP";
}


sub _name_default
{
    my $class = shift;
    $class = ref $class || $class;
    $class =~ s/^flo::plugin:://;
    $class =~ s/\.pm$//;
    $class = lc ($class);
    $class =~ s/::/./g;
    return ".$class";
}



# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

=head1 NOTES

B<Really> needs refactoring.

This class contains a lot of a pseudo-experimental code (whose purpose is to flesh
out ideas for a generic plugin architecture).  It is half procedural, half object
oriented due to the way plugins currently work.

Should be combined with flo::Plugin (which should be inherited)  -  A lot is
re-implemented here.

I'm not happy with the idea of $class->parent() - what happens if the
parent object ever changes?  And if plugin needs control of it's language?

Some of this code could be taken out to form a basis for all MKDoc objects.

A better design decision would be to return a list of states to process_request(),
which would call the listed methods until one returned a document.  And come to
think of it, states are something that should be achieved *after* we run all the
code.  Perhaps a state method to set current state...  This could really be
improved.

=head1 AUTHOR

Steve Purkis <spurkis@mkdoc.com>

=head1 VERSION

$Id: Generic.pm,v 1.10.2.16 2003/05/30 12:29:32 jhiver Exp $

=head1 COPYRIGHT

Copyright (c) 2002, MKDoc Holdings Ltd.  All rights reserved.

=head1 SEE ALSO

L<flo::Record::Ticket>,
L<flo::Record::Editor>,
L<flo::plugin::Account::*> releated plugins,
L<flo::plugin::explorer::Admin> contains similar functionality

=cut



1;


__END__
