# -----------------------------------------------------------------------------
# MKDoc::Test::Framework
# -----------------------------------------------------------------------------
#       Author : Steve Purkis <spurkis@mkdoc.com>
#    Copyright : Copyright (c) 2002 MKDoc Holdings Ltd.  All rights reserved.
#      Version : $Id: Framework.pm,v 1.1.2.10 2002/12/17 18:28:26 jhiver Exp $
#
#   Description:
#
#       This framework lets you use the t/site directory as if it was
#       both the MKDoc Home Directory, and an MKDoc Site installation.
#
#       It assumes that all tests are run from the build directory (which
#       contains a 't/site' directory).
#
#     Synopsis :
#
#       use lib 't/lib';
#
#       # you must use Test::Framework before MKDoc is loaded:
#       use MKDoc::Test::Framework;
#
# -----------------------------------------------------------------------------

package MKDoc::Test::Framework;

use strict;
use warnings;

use File::Spec;
use MKDoc::Site::Config;
use Carp;


##
# Package Variables
# =============================================================================
##

## MKDoc Home directory
our $MKDOC_DIR;

## MKDoc Site directory
our $SITE_DIR;

## Server name, found in $MKDOC_DIR/conf/sites.conf
our $SERVER_NAME;

##
# -----------------------------------------------------------------------------
# Override MKDOC_DIR and SERVER_NAME in the environment before MKDoc.pm
# is used.  This must happen first, otherwise wierd things might happen.
# -----------------------------------------------------------------------------
##
BEGIN
{
    $MKDOC_DIR       = File::Spec->canonpath('t/site');
    $SITE_DIR        = File::Spec->canonpath('t/site');
    $SERVER_NAME     = 'test';

    $ENV{MKDOC_DIR}      = $MKDOC_DIR;
    $ENV{MKDOC_SITE_DIR} = $SITE_DIR;
    $ENV{SERVER_NAME}    = $SERVER_NAME;
    $MKDoc::Site::Config::CONFIG_FILE = "$SITE_DIR/config/init.cfg";
};


##
# Class Methods
# -----------------------------------------------------------------------------
##


##
# $class->initialize;
# -------------------
# Imports MKDoc module and spawn database objects.
##
sub initialize
{
    eval <<EOF;
use MKDoc;
MKDoc::init ('cgi.mkdoc');
EOF

    defined $@ and $@ and confess ($@);
}


##
# $path = $class->mkdoc_dir
# ------------------------------
# Returns the test MKDoc directory, relative to the build directory.
##
sub mkdoc_dir { return $MKDOC_DIR; }


##
# $path = $class->site_dir
# -----------------------------
# Returns the test site directory, relative to the build directory.
##
sub site_dir { return $SITE_DIR; }


##
# $path = $class->site_setup_file
# -------------------------------
# Returns the test site's setup.cfg file, relative to the build directory.
##
sub site_setup_file { return MKDoc::Site::Config->config_file; }


##
# $path = $class->site_setup_config
# ---------------------------------
# Returns an MKDoc::Site::Config object containing the parsed site
# initialization file.
##
sub site_setup_config
{
    return new MKDoc::Site::Config();
}


1;

__END__
