# -----------------------------------------------------------------------------
# TestUtils - Utils for perl test scripts
# -----------------------------------------------------------------------------
#      Author  : Steve Purkis <spurkis@mkdoc.com>
#      Version : $Id: TestUtils.pm,v 1.1.2.4 2002/11/14 17:14:54 spurkis Exp $
#    Copyright : Copyright (c) 2002 MKDoc Holdings Ltd.  All rights reserved.
#
#    Description:
#       Contains commonly used subs to avoid code duplication in the test
#       suite.
#
#    Notes:
#       run_tests() has *a lot* in common with Test::Class (avail. on CPAN).
#       I would suggest avaluating it as a replacement.
#
#    Synopsis:
#       use lib 't/lib';
#       use TestUtils qw/ run_tests set_debug_from_argv /;
#
#       set_debug_from_argv( @classes );
#       run_tests(
#                 setup    => \&setup,
#                 tests    => \&tests,
#                 teardown => \&teardown,
#                );
#
#       TestUtils::stop_stdout();     # redirected to /dev/null
#       TestUtils::restart_stdout();
#
# -----------------------------------------------------------------------------

package TestUtils;

use strict;
use warnings;
use 5.008_000;

use File::Spec;
use Test::More;
use Exporter ();

use base qw/ Exporter /;


##
#  Package Variables
#  ============================================================================
##

##  Version (from CVS revision string).
our $VERSION = (split(/ /, '$Revision: 1.1.2.4 $'))[1];

##  Boolean switch controlling debug printing to STDERR.
our $DEBUG = 0;

##  Exportable items
our @EXPORT_OK = qw/ &run_tests &set_debug_from_argv /;



##
#  Subroutines
#  ============================================================================
##


##
# $debug = set_debug_from_argv( @classes )
# ----------------------------------------
#
# Checks for a -d switch in @ARGV.  If set, turns on the
# $DEBUG variable in this class, the 'main' package, and
# each class named in @classes.
#
# Returns the value of debug.
##
sub set_debug_from_argv
{
    my @classes = @_;

    my $debug = grep /-d/, @ARGV;
    if ($debug)
    {
	warn "  +  debug set to $debug\n";
	$DEBUG       = $debug;
	$main::DEBUG = $debug;

	foreach my $class (@classes)
	{
	    no strict 'refs';
	    ${$class . '::DEBUG'} = $debug;
	}
    }

    return $debug;
}


##
# $exit_status = run_tests( %args )
# ----------------------------------
# Runs named test in an eval block.  Valid arguments:
#
#       setup    => sub { ... }
#       tests    => sub { ... }
#       teardown => sub { ... }
#
# Only 'tests' is required.
#
# Execution logic:
#   1. 'setup' is executed first, if defined.
#      On failure: 'Test Setup Error' reported, returns with status 1.
#   2. 'tests' is executed second.
#      On failure: 'Test Error' reported, returns with status 2.
#   3. 'tests' is executed second.
#      On failure: 'Test Teardown Error' reported, returns with status 3.
#
# Failures are reported with Test::More's fail().
#
# Returns 0 on success, non-zero on failure.
##
sub run_tests
{
    my %args = @_;
    my $exit_status = 0;

    if (my $setup = $args{setup})
    {
	eval { &$setup; };
	if ($@)
	{
	    fail("Test Setup Error:");
	    diag($@);
	    return($exit_status = 1);
	}
    }

    if (my $tests = $args{tests})
    {
	eval { &$tests; };
	if ($@)
	{
	    fail("Test Error:");
	    diag($@);
	    return($exit_status = 2);
	}
    }
    else
    {
	diag("Test Error: No tests specified!\n");
    }

    if (my $teardown = $args{teardown})
    {
	eval { &$teardown; };
	if ($@)
	{
	    fail("Test Teardown Error:");
	    diag($@);
	    return($exit_status = 3);
	}
    }

    return $exit_status;
}


##
# stop_stdout()
# -------------
# Stops STDOUT (by redirecting it to File::Spec's /dev/null).
##
sub stop_stdout {
    unless ($TestUtils::_stdout) {
	my $dev_null = File::Spec->devnull;
	open ($TestUtils::_stdout, ">&STDOUT") || die "Test error: can't dup STDOUT: $!";
	open (STDOUT, '>', $dev_null)          || die "Can't redirect STDOUT to $dev_null: $!";
    }
}


##
# restart_stdout()
# ----------------
# Restarts a previously stopped STDOUT.
##
sub restart_stdout {
    if ($TestUtils::_stdout) {
	close STDOUT;
	open (STDOUT, ">&", $TestUtils::_stdout) || die "Can't dup \$TestUtils::_stdout: $!";
	undef $TestUtils::_stdout;
    }
}


##
# stop_stderr()
# -------------
# Stops STDERR (by redirecting it to /dev/null).
##
sub stop_stderr {
    unless ($TestUtils::_stderr) {
	my $dev_null = File::Spec->devnull;
	open ($TestUtils::_stderr, ">&STDERR") || die "Test error: can't dup STDERR: $!";
	open (STDERR, '>', $dev_null)          || die "Can't redirect STDERR to $dev_null: $!";
    }
}

##
# restart_stderr()
# ----------------
# Restarts a previously stopped STDERR.
##
sub restart_stderr {
    if ($TestUtils::_stderr) {
	close STDERR;
	open (STDERR, ">&", $TestUtils::_stderr) || die "Can't dup \$TestUtils::_stderr: $!";
	undef $TestUtils::_stderr;
    }
}



1;

__END__
