#!/usr/bin/perl
package mkdoc::find;

use strict;
use diagnostics;

use Exporter;
use vars qw($VERSION @ISA @EXPORT);

use constant VERSION => 0.91;

@ISA = qw(Exporter);
@EXPORT = qw(findfile);

use Getopt::Long;
use Cwd;
use File::Basename;
use File::Find;
use File::Spec::Functions qw(splitdir rel2abs);

my %findoptiondefault;
my %findoption;
my $startdepth;
my @found;

BEGIN
{
  %findoptiondefault = (
    finddebug           => 0,
    findfile            => 0,
    finddirectory       => 0,
    findtrim            => 0,
    findstart           => Cwd::cwd(),
    finddepth           => 0, # 0 implies --findrecurse; 1 for current folder
    findname            => '',
    findignorename      => '',
    findignoredirectory => '',
    findsize            => 0,
    finddays            => 0,
    findeval            => 0,
    findrecurse         => 1
  );

  Getopt::Long::Configure('pass_through');
  GetOptions(\%findoptiondefault,
    'finddebug',
    'findfile',
    'finddirectory',
    'findtrim',
    'findstart=s',
    'finddepth=i',
    'findname=s',
    'findignorename=s',
    'findignoredebug=s',
    'findsize=i',
    'finddays=i',
    'findeval=s',
    'findrecurse'
  );
}

sub findeval
{
  my ($file, $eval) = @_;
  my $found = 0;

  if ($findoption{findfile} && (-f $file)) {
    open(FH, '<' . $file)
      or die 'died:' . __PACKAGE__ . ": can't read $file: $!";

    while (<FH>) {
      $found = 1 if eval $eval;
      last if $found;
    }

    close(FH)
      or die 'died:' . __PACKAGE__ . ": can't close $file: $!";
  }
  elsif ($findoption{finddirectory} && (-d $file)) {
    opendir(DH, $file)
      or die 'died:' . __PACKAGE__ . ": can't read $file: $!";

    foreach (readdir(DH)) {
      $found = 1 if eval $eval;
      last if $found;
    }

    closedir(DH)
      or die 'died:' . __PACKAGE__ . ": can't close $file: $!";
  }

  return $found;
}

sub processfile
{
  my $wanted = 1;

  $wanted = 0 if (
    ($findoption{findfile}            && !(-f $File::Find::name)) ||
    ($findoption{finddirectory}       && !(-d $File::Find::name)) ||
    ($findoption{findname}            && (basename($File::Find::name) !~ /^$findoption{findname}$/)) ||
    ($findoption{findignorename}      && (-f $File::Find::name) && (basename($File::Find::name) =~ /^$findoption{findignorename}$/)) ||
    ($findoption{findignoredirectory} && (-d $File::Find::name) && (basename($File::Find::name) =~ /^$findoption{findignoredirectory}$/)) ||
    ($findoption{findsize}            && (-s $File::Find::name < $findoption{findsize})) ||
    ($findoption{finddays}            && (-M $File::Find::name > $findoption{finddays})) ||
    ($findoption{findeval}            && !(findeval($File::Find::name, $findoption{findeval})))
  );

  if ($wanted) {
    if ($findoption{findtrim}) {
      push @found, substr($File::Find::name, length($findoption{findstart}));
    }
    else {
      push @found, $File::Find::name;
    }
  }

  $File::Find::prune = 1
    if (
      ($findoption{findignoredirectory} &&
      (-d $File::Find::name) &&
      (basename($File::Find::name) =~ /^$findoption{findignoredirectory}$/))
      ||
      ($findoption{finddepth} &&
      (splitdir($File::Find::name) - $startdepth >= $findoption{finddepth})
      )    
    );
}

sub findfile
{
  my (%useroption) = @_;

  %findoption = %findoptiondefault;
  @found = ();

  while (my ($key, $value) = each %useroption) {
    warn 'warn: ' . __PACKAGE__ . ": unknown find option $key\n" if !(exists $findoption{$key});
    $findoption{$key} = $value;
  }

  # expand tildes (~), and relative paths to absolute paths
  $findoption{findstart} = rel2abs(glob($findoption{findstart}));

  # set the depth to current folder if we're not recursing
  $findoption{finddepth} = 1 if (!$findoption{findrecurse});

  if ($findoption{finddebug}) {
    while (my ($key, $value) = each %findoption) {
      warn 'debug: ' . __PACKAGE__ . ": find option $key -> $value\n";
    }
  }

  $startdepth = scalar splitdir($findoption{findstart});
  find(\&processfile, $findoption{findstart});

  return sort @found;
}

1;

