#!/usr/bin/perl
use strict;
use diagnostics;

use English;
use Pod::Usage;
use Getopt::Long;
use File::Spec::Functions qw(rel2abs);

use FindBin qw($Bin);
use lib "$Bin";
use mkdoc::find;

use constant DELIMITER =>
{
  BEGIN => '[[',
  END => ']]'
};

use constant FOLDER =>
{
  IGNORE => 'CVS'
};

use constant FILE =>
{
  TEMPLATE => 'xx\.html'
};

sub getoption
{
  my %optiondefault;

  %optiondefault = (
    help => 0,
    templatepath => '',
    outputfile => '',
    quiet => 0
  );

  GetOptions
  (
    \%optiondefault,
    'help|h',
    'templatepath|t=s',
    'outputfile|f=s',
    'quiet|q'
  )
    or pod2usage(2);

  pod2usage(1)
    if ($optiondefault{help});

  pod2usage("ERROR: No template path given.\n       ^")
    if (!($optiondefault{templatepath}) && (-t STDIN));

  pod2usage("ERROR: No output file given.\n       ^")
    if (!($optiondefault{outputfile}) && (-t STDIN));

  return %optiondefault;
}

sub readfile
{
  my ($filename) = @_;

  # expand tildes (~), and relative paths to absolute paths
  $filename = rel2abs(glob($filename));

  open(FH, '<' . $filename)
    or die "i can't read $filename: $!\n";

  my $content = join('', <FH>);

  close(FH)
    or die "i can't close $filename: $!\n";

  return ($content);
}

sub writefile
{
  my ($filename, @content) = @_;

  # STDOUT
  if (!$filename)
  {
    print @content;
    return 1;
  }

  # expand tildes (~), and relative paths to absolute paths
  $filename = rel2abs(glob($filename));

#  if (-e $filename)
#  {
#    print "Warning: $filename exists. Overwrite [y|N] ?";
#    my $in = <STDIN>;
#
#    chomp($in);
#
#    return 0 if (lc($in) ne 'y');
#  }

  open(FH, '>' . $filename)
    or die "i can't write $filename: $!\n";

  print FH @content;

  close(FH)
    or die "i can't close $filename: $!\n";

  return 1;
}

#in: template file
#out: hash of [[]]
sub readtemplate
{
  my ($template) = @_;

  my $BEGIN = DELIMITER->{BEGIN};
  my $END = DELIMITER->{END};

  my ($convert, %capture);

  $convert = readfile($template);

  while ($convert =~
    /
    (
    \Q$BEGIN\E # catch the start delimiter
    .*?        # grab the contents, and don't be greedy
    \Q$END\E   # catch the end delimiter
    )
    /gsmx
  )
  {
    $capture{$1} = 1;
  }

  return (%capture);
}

#in: templatepath
#out: hash
sub dotemplate
{
  my ($templatepath, $quiet) = @_;

  my %resource;

  my %findtemplate = (
    findignoredirectory => FOLDER->{IGNORE},
    findfile => 1,
    findname => FILE->{TEMPLATE},
    findstart => $templatepath
  );

  foreach my $template (findfile(%findtemplate))
  {
    print "Reading $template..." if (!$quiet);
    my %new = readtemplate($template);

    foreach my $key (keys %new)
    {
      $resource{$key} = 1;
    }

    print "OK\n" if (!$quiet);
  }

  return (%resource);
}

sub main
{
  my (%option, %hash, $done);

  %option = getoption();
  %hash = dotemplate($option{templatepath}, $option{quiet});

  foreach my $key (sort keys %hash)
  {
    $done .= "$key\n" . DELIMITER->{BEGIN} . DELIMITER->{END} . "\n\n";
  }

  writefile($option{outputfile}, $done);
}

main();

__END__

=pod

=head1 NAME

mkresource.pl - Create an empty resource file of strings harvested from generic templates.

=head1 SYNOPSIS

mkresource.pl --templatepath=<path> --outputfile=<file> [options]

Try --help for more help.

=head1 EXAMPLE

mkresource.pl -t mkdoc-site/templates -o mkdoc-site/templates/resource/xx.txt

=head1 OPTIONS

    -h   --help                  print this message
    -r   --templatepath=<path>   mkdoc template path
    -o   --outputfile=<file>     resource file to create
    -q   --quiet                 suppress non-critical error messages

=head1 DESCRIPTION

To create a resource file ready for translation, we recurse over the template path
collecting strings from all the generic templates. Duplicate translation keys are
discarded.

=over 8

=item -h --help

Print a brief help message and exits.

=item -t --templatepath=<path>

Specify the MKDoc template path to harvest strings from.

=item -o --outputfile=<file>

Specify the sorted, unique, empty resource file to create.

=item -q --quiet

Suppress non-critical messages.

=back

=cut

